@@ -4,7 +4,7 @@ Provides the Distribution class, which represents the module distribution
 being built/installed/distributed.
 """
 
-__revision__ = "$Id: dist.py 73123 2009-06-01 22:36:26Z tarek.ziade $"
+__revision__ = "$Id: dist.py 73836 2009-07-04 03:01:33Z tarek.ziade $"
 
 import sys, os, re
 
@@ -596,14 +596,14 @@ Common commands: (see '--help-commands' for more)
                 options = self.global_options
             parser.set_option_table(options)
             parser.print_help(self.common_usage + "\nGlobal options:")
-            self.announce('')
+            print('')
 
         if display_options:
             parser.set_option_table(self.display_options)
             parser.print_help(
                 "Information display options (just display " +
                 "information, ignore any commands)")
-            self.announce('')
+            print('')
 
         for command in self.commands:
             if isinstance(command, type) and issubclass(command, Command):
@@ -617,9 +617,9 @@ Common commands: (see '--help-commands' for more)
             else:
                 parser.set_option_table(klass.user_options)
             parser.print_help("Options for '%s' command:" % klass.__name__)
-            self.announce('')
+            print('')
 
-        self.announce(gen_usage(self.script_name))
+        print(gen_usage(self.script_name))
 
     def handle_display_options(self, option_order):
         """If there were any non-global "display-only" options
@@ -634,8 +634,8 @@ Common commands: (see '--help-commands' for more)
         # we ignore "foo bar").
         if self.help_commands:
             self.print_commands()
-            self.announce('')
-            self.announce(gen_usage(self.script_name))
+            print('')
+            print(gen_usage(self.script_name))
             return 1
 
         # If user supplied any of the "display metadata" options, then
@@ -651,12 +651,12 @@ Common commands: (see '--help-commands' for more)
                 opt = translate_longopt(opt)
                 value = getattr(self.metadata, "get_"+opt)()
                 if opt in ['keywords', 'platforms']:
-                    self.announce(','.join(value))
+                    print(','.join(value))
                 elif opt in ('classifiers', 'provides', 'requires',
                              'obsoletes'):
-                    self.announce('\n'.join(value))
+                    print('\n'.join(value))
                 else:
-                    self.announce(value)
+                    print(value)
                 any_display_options = 1
 
         return any_display_options
@@ -665,7 +665,7 @@ Common commands: (see '--help-commands' for more)
         """Print a subset of the list of all commands -- used by
         'print_commands()'.
         """
-        self.announce(header + ":")
+        print(header + ":")
 
         for cmd in commands:
             klass = self.cmdclass.get(cmd)
@@ -676,7 +676,7 @@ Common commands: (see '--help-commands' for more)
             except AttributeError:
                 description = "(no description available)"
 
-            self.announce("  %-*s  %s" % (max_length, cmd, description))
+            print("  %-*s  %s" % (max_length, cmd, description))
 
     def print_commands(self):
         """Print out a help message listing all available commands with a
@@ -907,8 +907,8 @@ Common commands: (see '--help-commands' for more)
 
     # -- Methods that operate on the Distribution ----------------------
 
-    def announce(self, msg, level=1):
-        log.debug(msg)
+    def announce(self, msg, level=log.INFO):
+        log.log(level, msg)
 
     def run_commands(self):
         """Run each command that was seen on the setup script command line.
